/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  midiAppendPropertylist.c
* Description:  Contains a routine to add a property list
*
* History:
* 04-June-07     (agabasch) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include "midiAppendPropertylist.h"
#include <config.h>
#include <string.h>

cpl_error_code appendPropertylist(const char *ProductFile, 
                                  cpl_frame_type type, 
                                  const char     *tag,
                                  cpl_frameset *frameset,
                                  cpl_parameterlist  *parlist)
{
   cpl_image  * pImage        = NULL;
   cpl_frame  * pFrameProduct = NULL;
   cpl_propertylist  *   pHeader = NULL;
   
   
   const char *szProduct=ProductFile;
   const char *szRawFile=ProductFile;
   
   
   pImage = cpl_image_load(szRawFile, CPL_TYPE_FLOAT, 0, 0 );
   
   
/*    pHeader = cpl_propertylist_new(); */
   pHeader = cpl_propertylist_load(szRawFile, 0 );
   
   
   pFrameProduct = cpl_frame_new();
   cpl_frame_set_filename( pFrameProduct, szProduct );
   cpl_frame_set_type    ( pFrameProduct, type );
   cpl_frame_set_tag     ( pFrameProduct, tag );
   cpl_frame_set_group   ( pFrameProduct, CPL_FRAME_GROUP_PRODUCT );
   cpl_frame_set_level   ( pFrameProduct, CPL_FRAME_LEVEL_FINAL );
   
   
//   cpl_frame_set_group(cpl_frameset_get_first(frameset), CPL_FRAME_GROUP_RAW) ;
   
   
#if defined CPL_VERSION_CODE && CPL_VERSION_CODE >= CPL_VERSION(4, 8, 0)   
   if (CPL_ERROR_NONE != cpl_dfs_setup_product_header(pHeader, pFrameProduct, frameset, parlist, "dummy-recid", PACKAGE "/" PACKAGE_VERSION, PRODUCT_DID, NULL))
   {
      cpl_msg_error(cpl_func,"Error in cpl_dfs_setup_product_header: %s %s",cpl_error_get_message(),cpl_error_get_where());
   }
#else
   if (CPL_ERROR_NONE != cpl_dfs_setup_product_header(pHeader, pFrameProduct, frameset, parlist, "dummy-recid", PACKAGE "/" PACKAGE_VERSION, PRODUCT_DID))
   {
      cpl_msg_error(cpl_func,"Error in cpl_dfs_setup_product_header: %s %s",cpl_error_get_message(),cpl_error_get_where());
   }
#endif

   
   /*   if (CPL_ERROR_NONE != irplib_image_save(frameset, */
/*                                            parlist, */
/*                                            frameset, */
/*                                            pImage, */
/*                                            CPL_BPP_IEEE_FLOAT, */
/*                                            "Test", */
/*                                            tag, */
/*                                            pHeader, */
/*                                            NULL, */
/*                                            "Test", */
/*                                            szProduct)) */
/*    { */
/*       cpl_msg_error(cpl_func,"Error in irplib_image_save: %s %s",tag,cpl_error_get_message()); */
/*    } */
   
   
   
   
   if (CPL_ERROR_NONE != cpl_image_save(pImage, szProduct, CPL_BPP_IEEE_FLOAT, pHeader, CPL_IO_DEFAULT ))
   {
      cpl_msg_error(cpl_func,"Error in cpl_image_save");
   }
   
   if (CPL_ERROR_NONE != cpl_frameset_insert( frameset, pFrameProduct ))
   {
      cpl_msg_error(cpl_func,"Error in cpl_frameset_insert");
   }
   
   cpl_image_delete( pImage );
   cpl_propertylist_delete(pHeader ); 
   
   return 0;
}

cpl_error_code appendPropertylist_table(const char   *ProductFile, 
                                        cpl_frame_type     type, 
                                        const char         *tag,
                                        cpl_frameset       *frameset,
                                        cpl_parameterlist  *parlist)
{
   
   int extnum,i;
   
/*    cpl_table         * pTable         = NULL; */
   cpl_table        ** pTable_ext  = NULL;
   
   cpl_propertylist  * pHeader        = NULL;
   cpl_propertylist ** pHeader_ext = NULL;
   
   cpl_frame         * pFrameProduct  = NULL;
   cpl_frame         * pFrame_tmp  = NULL;
   
   
   const char *szProduct=ProductFile;
   const char *szRawFile=ProductFile;
   char * current_extension;

   pFrameProduct = cpl_frame_new();
   cpl_frame_set_filename( pFrameProduct, szProduct );
   cpl_frame_set_type    ( pFrameProduct, type );
   cpl_frame_set_tag     ( pFrameProduct, tag );
   cpl_frame_set_group   ( pFrameProduct, CPL_FRAME_GROUP_PRODUCT );
   cpl_frame_set_level   ( pFrameProduct, CPL_FRAME_LEVEL_FINAL );
   
//   cpl_frame_set_group(cpl_frameset_get_first(frameset), CPL_FRAME_GROUP_RAW) ;


/* read the number of extensions from the rawfile */
   pFrame_tmp = cpl_frame_new();
   cpl_frame_set_filename( pFrame_tmp, szRawFile );
   extnum=cpl_frame_get_nextensions(pFrame_tmp);
   cpl_frame_delete(pFrame_tmp);
   
   pHeader = cpl_propertylist_load(szRawFile, 0 );


   if (pHeader ==NULL)
   {
      cpl_msg_error(cpl_func,"Error loading the propertylist of %s",szRawFile); 
      return -1;
   }
   
   pTable_ext  = cpl_malloc((extnum) * sizeof(cpl_table *));
   pHeader_ext = cpl_malloc((extnum) * sizeof(cpl_propertylist *));



   for (i=0; i<extnum; i++)
   {
      pHeader_ext[i] = cpl_propertylist_load(szRawFile, i+1 );
      pTable_ext[i]  = cpl_table_load(szRawFile, i+1, 1 );
      
      if (pHeader_ext[i] ==NULL || pTable_ext[i]==NULL)
      {
         cpl_msg_error(cpl_func,"Error loading the extended header or table of %s",szRawFile); 
         
         for (i=0; i<extnum; i++)
         {
            cpl_propertylist_delete(pHeader_ext[i]);
            cpl_table_delete(pTable_ext[i]); 
         }
         cpl_free (pHeader_ext);          
         cpl_free (pTable_ext);          
         
         
         return -1;
      }

/* ------------------------------------------------------------------- */
/*       OI_FITS standard requires to saves the FLAG column as boolean */
      current_extension=(char *)cpl_propertylist_get_string(pHeader_ext[i],"EXTNAME");
      
      if (strcmp(current_extension,"OI_VIS")==0 || strcmp(current_extension,"OI_VIS2")==0){
         
         if (cpl_table_has_column(pTable_ext[i],"FLAG")){
            cpl_table_set_column_savetype(pTable_ext[i], "FLAG", CPL_TYPE_BOOL);
         }
      }
      
/* ------------------------------------------------------------------- */
      
   }
   
#if defined CPL_VERSION_CODE && CPL_VERSION_CODE >= CPL_VERSION(4, 8, 0)   
   if (CPL_ERROR_NONE != cpl_dfs_setup_product_header(pHeader, pFrameProduct, frameset, parlist, "dummy-recid", PACKAGE "/" PACKAGE_VERSION, PRODUCT_DID, NULL))
   {
      cpl_msg_error(cpl_func,"Error in cpl_dfs_setup_product_header: %s %s",cpl_error_get_message(),cpl_error_get_where());
   }
#else
   if (CPL_ERROR_NONE != cpl_dfs_setup_product_header(pHeader, pFrameProduct, frameset, parlist, "dummy-recid", PACKAGE "/" PACKAGE_VERSION, PRODUCT_DID))
   {
      cpl_msg_error(cpl_func,"Error in cpl_dfs_setup_product_header: %s %s",cpl_error_get_message(),cpl_error_get_where());
   }
#endif


   if (CPL_ERROR_NONE != cpl_table_save(pTable_ext[0],pHeader, pHeader_ext[0], szProduct, CPL_IO_DEFAULT ))
   {
      cpl_msg_error(cpl_func,"Error when saving primary header: %s %s",cpl_error_get_message(),cpl_error_get_where());
   }

   for (i=1; i<extnum; i++)
   {
      if (CPL_ERROR_NONE != cpl_table_save(pTable_ext[i], NULL, pHeader_ext[i], szProduct, CPL_IO_EXTEND ))
      {
         cpl_msg_error(cpl_func,"Error when saving Extension: %s %s",cpl_error_get_message(),cpl_error_get_where());
      }

   }



   if (CPL_ERROR_NONE != cpl_frameset_insert( frameset, pFrameProduct ))
   {
      cpl_msg_error(cpl_func,"Error in cpl_frameset_insert");
   }
   

    cpl_propertylist_delete(pHeader);

   for (i=0; i<extnum; i++)
   {
      cpl_propertylist_delete(pHeader_ext[i]);
      cpl_table_delete (pTable_ext[i]); 
   }
   cpl_free (pTable_ext); 
   cpl_free (pHeader_ext); 
   

   return 0;  

}


cpl_error_code appendPropertylist_statistics(const char         *InputFile, 
                                             const char         *ProductFile, 
                                             cpl_frame_type     type, 
                                             const char         *tag,
                                             cpl_frameset       *frameset,
                                             cpl_parameterlist  *parlist,
                                             struct plots       *plotfiles_lambda,
                                             struct plots       *plotfiles_frames,
                                             struct plots       *plotfiles_scans,
                                             int                 dimen_lambda,
                                             int                 dimen_frames,
                                             int                 dimen_scans)
{
   
   int extnum,i;
   
/*    cpl_table         * pTable             = NULL; */
   cpl_table        ** pTable_ext         = NULL;
   
   cpl_propertylist  * pHeader            = NULL;
   cpl_propertylist ** pHeader_ext        = NULL;
   cpl_propertylist ** pHeader_ext_append = NULL;
   
   cpl_frame         * pFrameProduct      = NULL;
   cpl_frame         * pFrame_tmp         = NULL;
   
   
   const char *szProduct=ProductFile;
   const char *szRawFile=InputFile;
   char * current_extension;
   


/*    cpl_msg_info(cpl_func,"Entries: %s ",plotfiles_lambda[0].filename); */
/*    cpl_msg_info(cpl_func,"dimen_lambda: %d ",dimen_lambda); */
   

   cpl_vector *vec_lambda;
   cpl_vector *vec_frames;
   cpl_vector *vec_scans;
   
/*    cpl_vector  *vec; */

   cpl_table   *table_lambda; 
   cpl_table   *table_frames; 
   cpl_table   *table_scans; 
   

/* Determine the size of the Table */

   vec_lambda= vector_read(plotfiles_lambda[0].filename) ;
   table_lambda=cpl_table_new(cpl_vector_get_size(vec_lambda));
   cpl_vector_delete(vec_lambda);

   vec_frames= vector_read(plotfiles_frames[0].filename) ;
   table_frames=cpl_table_new(cpl_vector_get_size(vec_frames));
   cpl_vector_delete(vec_frames);

   vec_scans= vector_read(plotfiles_scans[0].filename) ;
   table_scans=cpl_table_new(cpl_vector_get_size(vec_scans));
   cpl_vector_delete(vec_scans);


/* Fill the Table */
   for(i=0; i<dimen_lambda; i++)   
   {
      vec_lambda= vector_read(plotfiles_lambda[i].filename) ;
      cpl_table_wrap_double(table_lambda, cpl_vector_get_data(vec_lambda), plotfiles_lambda[i].label) ;
      cpl_vector_unwrap(vec_lambda);
   }

   for(i=0; i<dimen_frames; i++)   
   {
      vec_frames= vector_read(plotfiles_frames[i].filename) ;
      cpl_table_wrap_double(table_frames, cpl_vector_get_data(vec_frames), plotfiles_frames[i].label) ;
      cpl_vector_unwrap(vec_frames);
   }

   for(i=0; i<dimen_scans; i++)   
   {
      vec_scans= vector_read(plotfiles_scans[i].filename) ;
      cpl_table_wrap_double(table_scans, cpl_vector_get_data(vec_scans), plotfiles_scans[i].label) ;
      cpl_vector_unwrap(vec_scans);
   }



/* read the number of extensions from the rawfile */
   pFrame_tmp = cpl_frame_new();
   cpl_frame_set_filename( pFrame_tmp, szRawFile );
   extnum=cpl_frame_get_nextensions(pFrame_tmp);
   cpl_frame_delete(pFrame_tmp);



   pFrameProduct = cpl_frame_new();
   cpl_frame_set_filename( pFrameProduct, szProduct );
   cpl_frame_set_type    ( pFrameProduct, type );
   cpl_frame_set_tag     ( pFrameProduct, tag );
   cpl_frame_set_group   ( pFrameProduct, CPL_FRAME_GROUP_PRODUCT );
   cpl_frame_set_level   ( pFrameProduct, CPL_FRAME_LEVEL_FINAL );
   
//   cpl_frame_set_group(cpl_frameset_get_first(frameset), CPL_FRAME_GROUP_RAW) ;


/*   cpl_msg_error(cpl_func,"extnum %d",extnum);     */


   pHeader = cpl_propertylist_load(szRawFile, 0 );
   if (pHeader ==NULL)
   {
      cpl_msg_error(cpl_func,"Error loading the propertylist of %s",szRawFile); 
      return -1;
   }
   
   pTable_ext  = cpl_malloc((extnum) * sizeof(cpl_table *));
   pHeader_ext = cpl_malloc((extnum) * sizeof(cpl_propertylist *));



   for (i=0; i<extnum; i++)
   {
      pHeader_ext[i] = cpl_propertylist_load(szRawFile, i+1 );
      pTable_ext[i]  = cpl_table_load(szRawFile, i+1, 1 );
      
      if (pHeader_ext[i] ==NULL || pTable_ext[i]==NULL)
      {
         cpl_msg_error(cpl_func,"Error loading the extended header or table of %s",szRawFile); 
         
         for (i=0; i<extnum; i++)
         {
            cpl_propertylist_delete(pHeader_ext[i]);
            cpl_table_delete(pTable_ext[i]); 
         }
            cpl_free (pHeader_ext);
            cpl_free (pTable_ext); 
         
         
         
         return -1;
      }
      
/* ------------------------------------------------------------------- */
/*       OI_FITS standard requires to saves the FLAG column as boolean */

      current_extension=(char *)cpl_propertylist_get_string(pHeader_ext[i],"EXTNAME");
      
      if (strcmp(current_extension,"OI_VIS")==0 || strcmp(current_extension,"OI_VIS2")==0){
         
         if (cpl_table_has_column(pTable_ext[i],"FLAG")){
            cpl_table_set_column_savetype(pTable_ext[i], "FLAG", CPL_TYPE_BOOL);
         }
      }
      
/* ------------------------------------------------------------------- */

   }
   
#if defined CPL_VERSION_CODE && CPL_VERSION_CODE >= CPL_VERSION(4, 8, 0)   
   if (CPL_ERROR_NONE != cpl_dfs_setup_product_header(pHeader, pFrameProduct, frameset, parlist, "dummy-recid", PACKAGE "/" PACKAGE_VERSION, PRODUCT_DID, NULL))
   {
      cpl_msg_error(cpl_func,"Error in cpl_dfs_setup_product_header: %s %s",cpl_error_get_message(),cpl_error_get_where());
   }
#else
   if (CPL_ERROR_NONE != cpl_dfs_setup_product_header(pHeader, pFrameProduct, frameset, parlist, "dummy-recid", PACKAGE "/" PACKAGE_VERSION, PRODUCT_DID))
   {
      cpl_msg_error(cpl_func,"Error in cpl_dfs_setup_product_header: %s %s",cpl_error_get_message(),cpl_error_get_where());
   }
#endif


   if (CPL_ERROR_NONE != cpl_table_save(pTable_ext[0],pHeader, pHeader_ext[0], szProduct, CPL_IO_DEFAULT ))
   {
      cpl_msg_error(cpl_func,"Error when saving primary header: %s %s",cpl_error_get_message(),cpl_error_get_where());
   }


   for (i=1; i<extnum; i++)
   {
      if (CPL_ERROR_NONE != cpl_table_save(pTable_ext[i], NULL, pHeader_ext[i], szProduct, CPL_IO_EXTEND ))
      {
         cpl_msg_error(cpl_func,"Error when saving Extension: %s %s",cpl_error_get_message(),cpl_error_get_where());
      }

   }



/* Create and append 3 new extensions */
   
   pHeader_ext_append = cpl_malloc(3 * sizeof(cpl_propertylist *));
   
   pHeader_ext_append[0]=cpl_propertylist_new();
   pHeader_ext_append[1]=cpl_propertylist_new();
   pHeader_ext_append[2]=cpl_propertylist_new();
   

   cpl_propertylist_append_string (pHeader_ext_append[0],"EXTNAME","STATISTICS1");
   cpl_propertylist_append_string (pHeader_ext_append[1],"EXTNAME","STATISTICS2");
   cpl_propertylist_append_string (pHeader_ext_append[2],"EXTNAME","STATISTICS3");
   
   if (CPL_ERROR_NONE != cpl_table_save(table_lambda, NULL, pHeader_ext_append[0], szProduct, CPL_IO_EXTEND ))
   {
      cpl_msg_error(cpl_func,"Error when saving Extension: %s %s",cpl_error_get_message(),cpl_error_get_where());
   }
   
   
   if (CPL_ERROR_NONE != cpl_table_save(table_frames, NULL, pHeader_ext_append[1], szProduct, CPL_IO_EXTEND ))
   {
      cpl_msg_error(cpl_func,"Error when saving Extension: %s %s",cpl_error_get_message(),cpl_error_get_where());
   }
   
   if (CPL_ERROR_NONE != cpl_table_save(table_scans, NULL, pHeader_ext_append[2], szProduct, CPL_IO_EXTEND ))
   {
      cpl_msg_error(cpl_func,"Error when saving Extension: %s %s",cpl_error_get_message(),cpl_error_get_where());
   }
   
   
   
   if (CPL_ERROR_NONE != cpl_frameset_insert( frameset, pFrameProduct ))
   {
      cpl_msg_error(cpl_func,"Error in cpl_frameset_insert");
   }
   
   
   /* Free the memory */
   
   cpl_table_delete(table_lambda);
   cpl_table_delete(table_frames);
   cpl_table_delete(table_scans);
   
   
   
   
   
   
   cpl_propertylist_delete(pHeader);
   
   for (i=0; i<extnum; i++)
   {
      cpl_propertylist_delete(pHeader_ext[i]);
      cpl_table_delete(pTable_ext[i]); 
   }
   cpl_free (pHeader_ext);
   cpl_free (pTable_ext); 
   
   
   cpl_propertylist_delete (pHeader_ext_append[0]);
   cpl_propertylist_delete (pHeader_ext_append[1]);
   cpl_propertylist_delete (pHeader_ext_append[2]);
   cpl_free(pHeader_ext_append);    
   
   
   
   return 0;  
   
}


/* cpl_vector_read modified to work wit one column: "%*d %lg" -> "%lg" */

/*----------------------------------------------------------------------------*/
/**
  @brief    Read a list of values from an ASCII file and create a cpl_vector
  @param    filename  Name of the input ASCII file
  @return   1 newly allocated cpl_vector or NULL in case of an error
  @see cpl_vector_dump

  Parse an input ASCII file values and create a cpl_vector from it
  Lines beginning with a hash are ignored, blank lines also.
  In valid lines the value is preceeded by an integer, which is ignored.

  The returned object must be deallocated using cpl_vector_delete()

  In addition to normal files, FIFO (see man mknod) are also supported.

  Possible #_cpl_error_code_ set in this function:
  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  - CPL_ERROR_FILE_IO if the file cannot be read
  - CPL_ERROR_BAD_FILE_FORMAT if the file contains no valid lines
 */
/*----------------------------------------------------------------------------*/
cpl_vector * vector_read(const char * filename)
{
    FILE       * in;
    cpl_vector * v;
    int          np = 0;
    int          size = 1000; /* Default size */
    char         line[1024];
    double       x;

    cpl_ensure(filename != NULL, CPL_ERROR_NULL_INPUT, NULL);

    /* Open the file */
    in = fopen(filename, "r");
    cpl_ensure(in != NULL, CPL_ERROR_FILE_IO, NULL);

    /* Create and fill the vector */
    v = cpl_vector_new(size);

    while (fgets(line, 1024, in) != NULL) {
        if (line[0] != '#' && sscanf(line, "%lg", &x) == 1) {
            /* Found new element
               - increase vector size if necessary,
               - insert element at end and
               - increment size counter */
            if (np == size) cpl_vector_set_size(v, size *= 2);
            cpl_vector_set(v, np++, x);
        }
    }

    /* Check that the loop ended due to eof and not an error */
    if (ferror(in)) {
        fclose(in);
        cpl_vector_delete(v);
        cpl_ensure(0, CPL_ERROR_FILE_IO, NULL);
    }

    fclose(in);

    /* Check that the file was not empty and set the size to its true value */
    if (np == 0 || cpl_vector_set_size(v, np)) {
        cpl_vector_delete(v);
        cpl_ensure(0, CPL_ERROR_BAD_FILE_FORMAT, NULL);
    }

    return v;
}
