/**********************************************************
***********************************************************
*				European Southern Observatory
*		     VLTI MIDI Data Reduction Software
*
* Module name:	midiFitsUtility.h
* Description:	
*
*
* History:		
* 23-Apr-03		(csabet) Created
***********************************************************
**********************************************************/
#ifndef _MIDI_FITS_UTILITY_H
#define _MIDI_FITS_UTILITY_H

/**********************************************************
*	Include files
**********************************************************/
#include "iauExchange.h"
#include "qfits.h"

/**********************************************************
*	Constant definitions
**********************************************************/

/**********************************************************
*	Type definitions 
**********************************************************/

/**********************************************************
*	Global Variables 
**********************************************************/

/**********************************************************
*	Function Prototypes
**********************************************************/
void selectChannels (
	int			start,
	int			end,
    ImageFormat	*format,
    float		**mask);

void selectMask (
    enum MaskMode	maskMode,
    MidiFiles		*fileNames,
    char			*maskFile,
    int				*error);

int getFitsExtensionNumber (
    char	*fileName,
    const char	*extName,
    int		*error);

void createMidiMaskFile (
    int			widthX,
    int			widthY,
    char		*inFileName,
    short int	*inData1,
    short int	*inData2,
    int			numOfFrames,
    int			subWindowSize,
    char		*maskFileName,
    int			*error);
	
void loadMidiMaskData (
    int		numOfFrames,
    int		subWindowSize,
    short	*inData1,
    short	*inData2,
    float	*outData1,
    float	*outData2,
    int		*error);

void writeMidiMaskFile (
    int		widthX,
    int		widthY,
    char	*maskFitsName,
    int		numOfElements,
    float	*data1,
    float	*data2,
    int		*error);

void writeMidiMaskImagData (
    int		iWidthX,
    int		iWidthY,
    char	*maskFitsName,
    int		numOfElements,
    int		numOfRows,
    float	*data1,
    float	*data2,
    int		*error);

void getFilterData (
    char		*fileName,
    FilterData	*filterInfo,
    int			*error);

int findImagingDataExtension (
    char	*fileName,
    int		defaultExtensionNumber,
    int		*error);

void getImageFormat (
    char    	*inFitsFile,
    int     	extNumOfImagingDataFile,
    ImageFormat *format,
    int			*error);

void midiInitFormat (
    ImageFormat *format);

void midiCopyFormat (
    ImageFormat *formatIn,
    ImageFormat *formatOut);

void midiReportFormat (
     const char		*type,
	int			isDisp,
    ImageFormat *format);

/*********************************************************/
#endif
/*********************************************************/
/*********************************************************/

