/* $Id: midi_dfs.c,v 1.11 2009-12-21 12:35:16 agabasch Exp $
 *
 * This file is part of the MIDI Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: agabasch $
 * $Date: 2009-12-21 12:35:16 $
 * $Revision: 1.11 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include <string.h>
#include <math.h>

#include <cpl.h>

#include "midi_dfs.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup midi_dfs  DFS related functions
 *
 * TBD
 */
/*----------------------------------------------------------------------------*/

/**@{*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Set the group as RAW or CALIB in a frameset
  @param    set     the input frameset
  @return   CPL_ERROR_NONE iff OK
 */
/*----------------------------------------------------------------------------*/
cpl_error_code midi_dfs_set_groups(cpl_frameset * set)
{
   cpl_errorstate prestate = cpl_errorstate_get();
   cpl_frame * frame = NULL;
   int         i = 0;
   
   int nframes = cpl_frameset_get_size(set);
   /* Loop on frames */

   for (i = 0; i < nframes; i++) {
       frame = cpl_frameset_get_position(set, i);
       const char * tag = cpl_frame_get_tag(frame);
      if (tag == NULL) {
         cpl_msg_warning(cpl_func, "Frame %d has no tag", i);
      } else if (!strcmp(tag, MIDI_HIGH_SENS_CALIB    ) ||
                 !strcmp(tag, MIDI_HIGH_SENS_SCIENCE  ) ||
                 !strcmp(tag, MIDI_SCI_PHOT_CALIB     ) ||
                 !strcmp(tag, MIDI_SCI_PHOT_SCIENCE   ) ||
                 !strcmp(tag, MIDI_KAPPA_SP_CALIB     ) ||
                 !strcmp(tag, MIDI_KAPPA_SP_SCIENCE   ) ||
                 !strcmp(tag, MIDI_ACQ                ) ||
                 !strcmp(tag, MIDI_DETLIN             ) ||
                 !strcmp(tag, MIDI_DETRON             ) ||
                 !strcmp(tag, MIDI_DSPTRN             ) ||
                 !strcmp(tag, MIDI_REFPIX             ) ||
                 !strcmp(tag, MIDI_WAVECAL            ) ||
                 !strcmp(tag, MIDI_PHOTOM_SP_CALIB    ) ||
                 !strcmp(tag, MIDI_PHOTOM_SP_SCIENCE  ) ||
                 !strcmp(tag, MIDI_PHOTOM_HS_CALIB    ) ||
                 !strcmp(tag, MIDI_PHOTOM_HS_SCIENCE  ) ||
                 !strcmp(tag, MIDI_OTHER_DETLIN       ) ||
                 !strcmp(tag, MIDI_OTHER_DETRON       ) ||
                 !strcmp(tag, MIDI_OTHER_DSPTRN       ) ||
                 !strcmp(tag, MIDI_OTHER_GENERIC      ) ||
                 !strcmp(tag, MIDI_OTHER_REFPIX       ) ||
                 !strcmp(tag, MIDI_OTHER_WAVECAL      ) ||
                 !strcmp(tag, MIDI_OTHER_ACQ          ) ||
                 !strcmp(tag, MIDI_DOME_AOPEN         ) ||
                 !strcmp(tag, MIDI_DOME_BOPEN         ) ||
                 !strcmp(tag, MIDI_INTERNAL_OPD       ) ||
                 !strcmp(tag, MIDI_CAL_SEARCH         ) ||
                 !strcmp(tag, MIDI_OTHER_FRINGE       )) 
      {
         /* RAW frames */
         cpl_frame_set_group(frame, CPL_FRAME_GROUP_RAW);
      } else if (!strcmp(tag, MIDI_CALIB_FLAT)          ||
                 !strcmp(tag, MIDI_KAPPA_SP_MASK_PRISM) ||
                 !strcmp(tag, MIDI_KAPPA_SP_MASK_GRISM) ||
                 !strcmp(tag, MIDI_KAPPA_HS_MASK_PRISM) ||
                 !strcmp(tag, MIDI_KAPPA_HS_MASK_GRISM) ||
                 !strcmp(tag, MIDI_KAPPAMATRIX_PRISM)   ||
                 !strcmp(tag, MIDI_KAPPAMATRIX_GRISM)   ||
                 !strcmp(tag, MIDI_TRF_GRISM)           ||
                 !strcmp(tag, MIDI_TRF_PRISM)           ||
                 !strcmp(tag, MIDI_CALIB_DATABASE))
      {
         /* CALIB frames */
         cpl_frame_set_group(frame, CPL_FRAME_GROUP_CALIB);
      }
      
      





   }
   
    if (!cpl_errorstate_is_equal(prestate)) {
        return cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                     "Could not identify RAW and CALIB "
                                     "frames");
    }

    return CPL_ERROR_NONE;
}

/**@}*/
