/* $Id: midi_detlin.c,v 1.6 2010-05-28 09:16:01 agabasch Exp $
 *
 * This file is part of the MIDI Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: agabasch $
 * $Date: 2010-05-28 09:16:01 $
 * $Revision: 1.6 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <stdio.h>
#include "midi_utils.h"
#include "midi_pfits.h"
#include "midi_dfs.h"
#include "midiControl.h"
#include "midiGlobal.h"
#include "midiAppendPropertylist.h"


/*-----------------------------------------------------------------------------
                            Functions prototypes
 -----------------------------------------------------------------------------*/

static int midi_detlin_create(cpl_plugin *) ;
static int midi_detlin_exec(cpl_plugin *) ;
static int midi_detlin_destroy(cpl_plugin *) ;
static int midi_detlin(cpl_parameterlist *, cpl_frameset *) ;

/*-----------------------------------------------------------------------------
                            Static variables
 -----------------------------------------------------------------------------*/

static char midi_detlin_description[] =
"The purpose of this technical template is to evaluate the linearity of the\n"
"detector, i.e. the relationship between the detector counts and the number\n"
"of photons.\n\n"
"Input files:\n\n"
"  DO category:               Type:       Explanation:         Required:\n"
"  DETLIN                     Raw         Raw data frame          Y\n\n"
"Output files:\n\n"
"  DO category:               Data type:  Explanation:\n"
"  REDUCED_DETLIN             FITS image  linearity coefficient in the header\n"
"\n";

/*-----------------------------------------------------------------------------
                                Functions code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Build the list of available plugins, for this module. 
  @param    list    the plugin list
  @return   0 if everything is ok, -1 otherwise

  Create the recipe instance and make it available to the application using the 
  interface. This function is exported.
 */
/*----------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
    cpl_recipe  *   recipe = cpl_calloc(1, sizeof *recipe ) ;
    cpl_plugin  *   plugin = &recipe->interface ;

    cpl_plugin_init(plugin,
                    CPL_PLUGIN_API,
                    MIDI_BINARY_VERSION,
                    CPL_PLUGIN_TYPE_RECIPE,
                    "midi_detlin",
                    "Evaluates the detector linearity",
                    midi_detlin_description,
                    "Coorosh Sabet",
                    PACKAGE_BUGREPORT,
                    midi_get_license(),
                    midi_detlin_create,
                    midi_detlin_exec,
                    midi_detlin_destroy) ;

    cpl_pluginlist_append(list, plugin) ;
    
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Setup the recipe options    
  @param    plugin  the plugin
  @return   0 if everything is ok

  Defining the command-line/configuration parameters for the recipe.
 */
/*----------------------------------------------------------------------------*/
static int midi_detlin_create(cpl_plugin * plugin)
{
    cpl_recipe      * recipe ;
/*    cpl_parameter   * p ;*/

    /* Check that the plugin is part of a valid recipe */
    if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE) 
        recipe = (cpl_recipe *)plugin ;
    else return -1 ;

    /* Create the parameters list in the cpl_recipe object */
    recipe->parameters = cpl_parameterlist_new() ; 
    
    /* Fill the parameters list */
/*     p = cpl_parameter_new_value("midi.midi_detlin.productDir",  */
/* 		CPL_TYPE_STRING, "Product directory", "midi.midi_detlin", */
/* 		"product path. don't forget ending /"); */
/*     cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "stropt1") ; */
/*     cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV) ; */
/*     cpl_parameterlist_append(recipe->parameters, p) ; */

/*
 	p = cpl_parameter_new_value("midi.midi_detlin.plotDuration", 
		CPL_TYPE_INT, "Plot duration (-1, 0, .. 5), -1 is the prompt mode", "midi.midi_detlin", 0);
    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "intopt1") ;
    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV) ;
    cpl_parameterlist_append(recipe->parameters, p) ;
*/


    /* Return */
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Execute the plugin instance given by the interface
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_detlin_exec(cpl_plugin * plugin)
{
    cpl_recipe  *   recipe ;
    
    /* Get the recipe out of the plugin */
    if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE) 
        recipe = (cpl_recipe *)plugin ;
    else return -1 ;

    batchNumber=0;
    return midi_detlin(recipe->parameters, recipe->frames) ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Destroy what has been created by the 'create' function
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_detlin_destroy(cpl_plugin * plugin)
{
    cpl_recipe  *   recipe ;
    
    /* Get the recipe out of the plugin */
    if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE) 
        recipe = (cpl_recipe *)plugin ;
    else return -1 ;

    cpl_parameterlist_delete(recipe->parameters) ; 
    return 0 ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    parlist     the parameters list
  @param    frameset   the frames list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_detlin(
	cpl_parameterlist	*parlist, 
	cpl_frameset		*frameset)
{
/* 	const char		*fctid = "midi_detlin" ; */
/*	cpl_parameter	*param ;*/
	cpl_frame		*current_frame;
	int				error=0;
	FILE			*sofPtr=NULL;
/* 	char 			*productDir;	// In: Directory path of products */
	int				plotDuration;

    /* RETRIEVE INPUT PARAMETERS */
/*     param = cpl_parameterlist_find(parlist, "midi.midi_detlin.productDir"); */
/*     productDir = (char *) (cpl_parameter_get_string(param)); */
/* 	printf ("productDir = %s \n", productDir); */
/*     if (cpl_error_get_code()) */
/* 	{ */
/*         cpl_msg_error(fctid, "Failed to retrieve productDir") ; */
/*         return -1 ; */
/*     } */

/*
    param = cpl_parameterlist_find(parlist, "midi.midi_detlin.plotDuration");
    plotDuration = cpl_parameter_get_int(param);
	printf ("plotDuration = %d \n", plotDuration);
    if ((plotDuration < -1) || (plotDuration > 5)) plotDuration = 0;
*/

    plotDuration = 0;

    cpl_frameset_iterator * it = cpl_frameset_iterator_new(frameset);
    current_frame = cpl_frameset_iterator_get(it);

    sofPtr = fopen ("MIDI_sof.log", "w");
	while ( current_frame && sofPtr )
	{	
		fprintf (sofPtr, "%s \n", (char *)cpl_frame_get_filename( current_frame ));
        cpl_frameset_iterator_advance(it, 1);
        current_frame = cpl_frameset_iterator_get(it);
    } /* All frames from frameset */
	fclose (sofPtr);
	cpl_frameset_iterator_delete(it);
    /* NOW PERFORMING THE DATA REDUCTION */
	executeDataReduction ("", "", "./", plotDuration, "MIDI_sof.log", &error,parlist,frameset);
/* 	executeDataReduction ("", "", productDir, plotDuration, "MIDI_sof.log", &error); */
	if (error) return -1;
	remove ("MIDI_sof.log");


        if (CPL_ERROR_NONE !=  appendPropertylist("MIDI_b1_lin.pro.fits", CPL_FRAME_TYPE_IMAGE, "REDUCED_DETLIN",frameset,parlist))
        {
           cpl_msg_error(cpl_func,"Error in appendPropertylist");
        }


    /* Return */
    if (cpl_error_get_code()) 
        return -1 ;
    else 
        return 0 ;
}
